#!
import tensorflow as tf
import tensorflow_datasets as tfds

dataset, metadata = tfds.load('fashion_mnist', as_supervised=True, with_info=True)
print(metadata)
train_dataset, test_dataset = dataset['train'], dataset['test']
train_dataset = train_dataset.shuffle(100).batch(12).repeat()

for img, label in train_dataset.take(1):
    img = img.numpy()
    print(img.shape)
    print(img)



import pandas as pd
import numpy as np
from Spectrum_analysis import spectrum
from Envelope_spectrum import envelope
from Power_spectrum_density import power
import pdb
import matplotlib.pylab as plt

# 读取文件
x = pd.read_csv('./122.csv')
x = np.array(x)             # 将读取的文件转换为数组
#x = np.delete(x,0,axis=0)   # 去掉第一行
x = np.delete(x,1,axis=1)   # 去掉第二列
x = x.astype(np.float64)    # 格式转为浮点数
x = np.ndarray.flatten(x)   # 将矩阵转换为一维数组

# 采样频率Fs=25600,信号长度L=32768
# 频谱分析
S = spectrum(x,25600,32768)
print("Spectrum_analysis:")
print(S[0])
print(S[1])

# 包络谱分析
E = envelope(x,25600,32768)
print("Envelope_spectrum:")
print(E[0])
print(E[1])

# 功率谱分析
P = power(x,25600,32768)
print("Power_spectrum_density :")
print(P[0])
print(P[1])




from hive_service import ThriftHive
from hive_service.ttypes import HiveServerException
from thrift import Thrift
from thrift.transport import TSocket
from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol
import subprocess

from pyarrow.parquet import ParquetDataSet
import pyarrow.parquet as pq
import pyarrow as pa
from libraries.hive_metastore.ThriftHiveMetastore import Client


def connect_hive() -> Client:
    """
    通过thrift连接hive metastore服务端
    """
    transport = TSocket.TSocket(host, int(port))
    transport = TTransport.TBufferedTransport(transport)
    transport.open()
    protocol = TBinaryProtocol.TBinaryProtocol(transport)
    return ThriftHiveMetastore.Client(protocol)

def write_table(client: Client, database: str, table: str, dataframe: DataFrame, partitions: list = None):
    """
    提供给用户将dataFrame写入hive表中的方式

    Examples:
        client = connect_hive(host, port)
         df = pd.DataFrame({
            'index': [1, 2, 3],
            'name': ['xiaoming', 'xiaowang', 'xiaozhang'],
            'prt_dt': ['2020', '2019', '2020']
        })

        partition_cols = ['prt_dt']
        write_table(client, database, table, df, partition_cols)
    
    Args:
    client(Client):hive客户端，通过thrift协议访问hive metastore
    database(str):数据库
    table(str):表名
    dataframe(pandas.DataFrame):pandas.DataFrame
    partitions(list):分区信息

    raise:
        HiveDatabaseNOTEXIST:Hive库不存在时抛出异常
        HiveTableNOTEXIST:Hive表不存在时抛出异常

    """
    # 1、连接hive服务端
    client = connect_hive(host, port)

    # 2、检查数据库是否存在，如果不存在则抛出异常
    databases = client.get_all_databases()
    if database not in databases:
        raise HiveDatabaseNOTEXIST('Hive database is not exist.')

    # 3、创建hive表，如果表名重复则抛出异常
    tables = client.get_all_tables(database)
    if table not in tables:
        raise HiveTableNOTEXIST('Hive table is not exist.')

    # 4、将pandas中字段int64类型转为int
    columns = dataframe.columns
    int64_fields = {}
    float64_fields = {}
    for field in columns:
        if pd.api.types.is_int64_dtype(dataframe[field]):
            int64_fields[field] = 'int32'

        if pd.api.types.is_float_dtype(dataframe[field]):
            float64_fields[field] = 'float32'
    transfer_fields = dict(int64_fields, **float64_fields)
    transfer_df = dataframe.astype(transfer_fields)

    # 5、将dataframe写入hive表中
    table_hdfs_path = client.get_table(database, table).sd.location
    table = pa.Table.from_pandas(transfer_df)
    pq.write_to_dataset(
        table=table, root_path=table_hdfs_path, partition_cols=partitions)

    # 6、写入分区表时需刷新元数据信息(msck repair table ***)
    shell = "hive -e 'msck repair table {}' ".format('train_data.telecom_train')
    subprocess.Popen(shell,shell=True)

def read_table(data_source: DataSource, database: str, table: str, partitions: list = None) -> DataFrame:
    """
    提供给用户根据hive库名和表名访问数据的方式-->dataframe（thrift、urllib、pyarrow、pyhdfs）

   Examples:
        client = connect_hive(host, port)
        read_table(client,'test','test')

    Args:
        client(Client):hive客户端，通过thrift协议访问hive metastore
        database(str):hive库名
        table(str):hive表名
        partitions(list):hive表分区（用户需按照分区目录填写）,如果查询所有数据，则无需填写分区

    Return:
        pandas.dataframe
    """
    # 1、连接hive服务端
    client = connect_hive(host, port)

    # 2、查询hive表元数据
    table = client.get_table(database, table)
    table_hdfs_path = table.sd.location

    logging.info('table_hdfs_path:' + table_hdfs_path)
    print(table_hdfs_path)

    # 3、判断hive是否为分区表,当用户没有输入partitions时需查找所有分区数据
    if partitions is not None:
        table_hdfs_path = [
            table_hdfs_path + constant.FILE_SEPARATION + x for x in partitions][0]
        dataframe = pq.ParquetDataset(
            table_hdfs_path).read().to_pandas()
        # pyarrow访问分区目录时，dataframe不含分区列，因此需添加分区列信息
        for partition in partitions:
            index = partition.find('=')
            field = partition[:index]
            field_value = partition[index + 1:]
            dataframe[field] = field_value
    else:
        dataframe = pq.ParquetDataset(
            table_hdfs_path).read().to_pandas()
    return dataframe



#!encoding=utf-8

'''
波谱分析，含频率谱，包络谱，功率谱分析，将现有代码封装为一类后，供给web侧调用
输入数据为[(),(),()...()]格式,生成器
输出数据为[(),(),()...()]格式,生成器

'''

import pandas as pd
import numpy as np
from Spectrum_analysis import spectrum
from Envelope_spectrum import envelope
from Power_spectrum_density import power
import pdb
import matplotlib.pylab as plt

class SpectroscopyAnalysis(object):
    '''
    对外接口类
    '''

    def __init__(self):
        pass

    def spectrum(self, x, fs=25600, l=32768):
        '''
        x 待分析数据
        fs 采样频率
        l 采样时间
        '''
        S = spectrum(x,fs,l)
        return S

    def envelope(self, x, fs=25600, l=32768):
        '''
        x 待分析数据
        fs 采样频率
        l 采样时间
        '''
        E = envelope(x,fs,l)
        return E

    def power(self, x, fs=25600, l=32768):
        '''
        x 待分析数据
        fs 采样频率
        l 采样时间
        '''
        P = power(x,fs,l)
        return P





# -*- coding: utf-8 -*-
from pyhive import hive

conn = hive.Connection(host='192.168.50.11', port=10000, username='', database='iptv')
cursor = conn.cursor()
cursor.execute('show tables')
#cursor.execute('select * from demo_table limit 10')
#for result in cursor.fetchall():
#    print result



'''
hive read helper

use pyarrow+thrift to read and write hive, choose this function is because
it's speed is faster than the other pkg-tools pyhive and impyla 

author: qin hai ning
date: 2021-03-25
'''


class HiveHelper(object):
    '''
    hive control of insert delete update query
    '''
    def __init__(self):
        pass

    def connect(self):




#包络谱分析
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from scipy.signal import hilbert
import pdb

def envelope(x,Fs,L):
    T = 1/Fs                    # 采样时间
    t = np.arange(L)/Fs         # 时间向量

    y = x-np.mean(x)            # 零均值化
    # 画出信号时域图像
    fig = plt.figure()
    fig.tight_layout()#调整整体空白
    plt.subplots_adjust(wspace =0, hspace =.5)#调整子图间距
    f = fig.add_subplot()
    f.plot(t,y)
    f.set_ylim(-25,25)
    f.set_xlabel('time (seconds)')
    #fig.show()

    # scipy.signal.hilbert,输入x是信号数据，返回值为hilbert变换后的解析信号
    xh = hilbert(x)
    # AM Demodulation
    xhAM = abs(xh)

    xhAM=xhAM - np.mean(xhAM);   # 零均值化
    NFFT = L
    # np.fft.fft，对输入信号进行快速傅里叶变换,输出长度与输入相同
    Y = np.fft.fft(xhAM)
    Y = Y[range(int(L/2)+1)]     # 单边频谱,由于对称性，只取一半区间
    A = 2 * abs(Y)/L             # 转换为幅值
    f = Fs/2*np.linspace(0, 1, NFFT//2+1)   # 横坐标，频率

    # 绘图显示结果
    fig1 = plt.figure()
    fig1.tight_layout()#调整整体空白
    plt.subplots_adjust(wspace =0, hspace =.5)#调整子图间距
    f1 = fig1.add_subplot(211)
    f1.plot(f,A)
    f1.set_xlabel('Frequency (Hz)')
    f1.set_title('Envelope spectrum')
    f2 = fig1.add_subplot(212)
    f2.plot(f[range(0,1000)], A[range(0,1000)])
    f2.set_xlabel("Freq(Hz)")
    f2.set_title('Envelope spectrum')
    #fig1.show()
    #pdb.set_trace()

    return  f,A    




import pandas as pd
from test_data import test_data
from __init__ import SpectroscopyAnalysis
import unittest
import pdb

class TestSetUp(unittest.TestCase):
    # 前置条件当中
    # 每一个测试用例方法执行之前都会运行的代码
    # 可以把测试数据放到 setUp 当中

    def setUp(self):
        print(test_data[:3])
        self.data = [[float(j) for j in i.split(",")] for i in test_data]
        print("\n===start test\n")

    def test_run(self):
        sa = SpectroscopyAnalysis()
        assert(type(sa.spectrum(self.data)) == tuple)
        assert(type(sa.envelope(self.data)) == tuple)
        assert(type(sa.power(self.data)) == tuple)

    def tearDown(self):
        print("\n===end test\n")

if __name__ == "__main__":
    unittest.main()
else:
    unittest.main()


